@define-color boxes_bg_color shade (@theme_bg_color, 0.5);
@define-color boxes_bg2_color mix (@theme_bg_color, @boxes_bg_color, 0.5);
@define-color boxes_selected_color #3465a4;

.boxes-snapshot-list-row {
    border-bottom: 1px solid @borders;
}

.boxes-snapshot-list-row.indicator {
    background-color: @insensitive_fg_color;
    border: none;
}

.boxes-snapshot-list-row.indicator.active {
    background-color: @theme_fg_color;
}

.boxes-resource-graph {
    background-color: @boxes_bg_color;
    border-color: @theme_fg_color;
    color: @theme_bg_color;
}

.transparent-bg {
    background-color: transparent;
    background-image: none;
}

.content-bg {
    background-color: @content_view_bg;
}

.boxes-toplevel {
    background-color: @theme_bg_color;
}

.slow-spinner {
    animation-duration: 2s;
}

.thumbnail {
    background: @boxes_bg2_color;
    border: 1px solid @theme_bg_color;
}

.bold-label {
    font-weight: bold;
}

/******************
 * New Box Wizard *
 ******************/

/* This is the subtitle in the ISOs/media list */
.boxes-step-label {
    font-size: smaller;
    font-weight: normal;
}

.boxes-wizard-media-os-label {
    color: #d8d8d8;
    font-weight: bold;
}

.boxes-logo-notice-label {
    font-size: smaller;
    font-style: italic;
}

.boxes-spice-tools-notice-label {
    font-size: smaller;
    font-style: italic;
}

.boxes-product-key-entry {
    font-family: monospace;
}

/* Ensure the viewport is transparent so the background don't
   show through the rounded corners */
.boxes-menu GtkViewport {
    background-image: none;
    background-color: transparent;
    border-style: none;
}

.boxes-menu-row {
    background: @theme_unfocused_base_color;
}

.boxes-menu-row:hover {
    background-color: @theme_bg_color;
}

.sources-list {
    border-radius: 5px;
}

separator {
    background-color: @borders;
}

.list-expand-button {
    border-top: 1px solid @borders;
}

/* Adds a border to the ISOs lists top undershoot */
.boxes-menu-scrolled.undershoot.top { border-top: 1px solid @borders; }

/* Screenshot flash */
@keyframes flash_animation {
    50% {
        border: 100px solid #FFF;
        opacity: 0.2;
    }
}

.flash {
    border: 1px solid transparent;
    animation: flash_animation;
    animation-duration: 150ms;
    animation-iteration-count: 1;
}

.overlay-box {
    padding: 25px;
    border-radius: 5px;
}

/* Welcome Tutorial */
.tutorial-page {
    background-repeat: no-repeat;
    background-size: 60%;
    background-position: center 15%;
}

.welcome-tutorial-title-label, .welcome-tutorial-description-label {
    color: white;
}
